/*
 *  Blocks.h
 *  Blocks
 *
 *  Created by Jesse Grosjean on 12/3/04.
 *  Copyright 2004 Hog Bay Software. All rights reserved.
 *
 */
#define	kASCPROVIDERASC	"com.apokalypsesoftware"
#define	kASCPROVIDER_APPMORI	kASCPROVIDERASC ".mori"

#import <Cocoa/Cocoa.h>

#import <Blocks/BKLog.h>
#import <Blocks/BKPluginRegistry.h>
#import <Blocks/BKPlugin.h>
#import <Blocks/BKExtensionPoint.h>
#import <Blocks/BKExtension.h>
#import <Blocks/BKRequirement.h>

#pragma	mark	general programming macros

#if	!defined(countof)
#	define	countof(a)	(sizeof(a) / sizeof(a[0]))
#endif

#pragma mark	general cocoa programming macros

#if	!defined(ObjectIsNil)
#	define	BKObjectIsNil(o)								((o) == nil)
#endif

#if	!defined(forall)
#	define	forall(collection, element)						for(id _ ## element ## _enumerator = [collection objectEnumerator], element; (element = [_ ## element ## _enumerator nextObject]) != nil; )
#endif

#if	!defined(forallby)
#	define	forallby(collection, element, enumerator)		for(id _ ## element ## _enumerator = [collection enumerator], element; (element = [_ ## element ## _enumerator nextObject]) != nil; )
#endif

#if	!defined(aftall)
#	define	aftall(collection, element)						for(id _ ## element ## _enumerator = [collection reverseObjectEnumerator], element; (element = [_ ## element ## _enumerator nextObject]) != nil; )
#endif

#pragma mark	cocoa memory management support macros

#define BKObjectAssign(assignee, object)					do { BKObjectReleaseIfNotNil(assignee); assignee = object; BKObjectRetainIfNotNil(assignee); } while(0)
#define BKObjectAssignInitial(assignee, object)				do { assignee = object; BKObjectRetainIfNotNil(assignee); } while(0)
#define BKObjectAssignInitialOnly(assignee, object)			do { assignee = object; } while(0)
#define BKObjectAssignInitialOwned(assignee, object)		BKObjectAssignInitialOnly(assignee, object)
#define BKObjectAssignOnly(assignee, object)				do { BKObjectReleaseIfNotNil(assignee); assignee = object; } while(0)
#define BKObjectAssignOwned(assignee, object)				do { BKObjectReleaseIfNotNil(assignee); assignee = object; } while(0)
#define BKObjectRetainIfNotNil(object)						do { if(object != nil) [object retain]; } while(0)
#define BKObjectRetain(object)								[object retain]
#define BKObjectRelease(assignee)							do { BKObjectReleaseIfNotNil(assignee); assignee = nil; } while(0)
#define BKObjectReleaseLater(assignee)						[assignee autorelease]
#define BKObjectReleaseIfNotNil(object)						do { if(object != nil) [object release]; } while(0)
#define BKObjectReleaseTemporary(object)					BKObjectReleaseIfNotNil(object)

#pragma	mark	localization support macros

#define BKLocalizedString(key, comment) [[NSBundle bundleForClass:[self class]] localizedStringForKey:(key) value:@"" table:nil]
#define BKLocalizedStringFromTable(key, tbl, comment) [[NSBundle bundleForClass:[self class]] localizedStringForKey:(key) value:@"" table:(tbl)]
#define BKLocalizedStringFromTableInBundle(key, tbl, bundle, comment) [bundle localizedStringForKey:(key) value:@"" table:(tbl)]
#define BKLocalizedStringWithDefaultValue(key, tbl, bundle, val, comment) [bundle localizedStringForKey:(key) value:(val) table:(tbl)]

#pragma	mark	clang static-analyzer annotations

#ifndef	__has_feature				// Optional.
#	define	__has_feature(x)	0	// Compatibility with non-clang compilers.
#endif

#ifndef	NS_RETURNS_RETAINED
#	if	__has_feature(attribute_ns_returns_retained)
#		define	NS_RETURNS_RETAINED		__attribute__((ns_returns_retained))
#	else
#		define	NS_RETURNS_RETAINED
#	endif
#endif

#ifndef	NS_RETURNS_NOT_RETAINED
#	if	__has_feature(attribute_ns_returns_not_retained)
#		define	NS_RETURNS_NOT_RETAINED	__attribute__((ns_returns_not_retained))
#	else
#		define	NS_RETURNS_NOT_RETAINED
#	endif
#endif

#ifndef	CF_RETURNS_RETAINED
#	if	__has_feature(attribute_cf_returns_retained)
#		define	CF_RETURNS_RETAINED		__attribute__((cf_returns_retained))
#	else
#		define	CF_RETURNS_RETAINED
#	endif
#endif

#ifndef	CF_RETURNS_NOT_RETAINED
#	if	__has_feature(attribute_cf_returns_not_retained)
#		define	CF_RETURNS_NOT_RETAINED	__attribute__((cf_returns_not_retained))
#	else
#		define	CF_RETURNS_NOT_RETAINED
#	endif
#endif

#ifndef	NS_CONSUMED
#	if	__has_feature(attribute_ns_consumed)
#		define	NS_CONSUMED				__attribute__((ns_consumed))
#	else
#		define	NS_CONSUMED
#	endif
#endif

#ifndef	CF_CONSUMED
#	if	__has_feature(attribute_cf_consumed)
#		define	CF_CONSUMED				__attribute__((cf_consumed))
#	else
#		define	CF_CONSUMED
#	endif
#endif

#ifndef	NS_CONSUMES_SELF
#	if	__has_feature(attribute_ns_consumes_self)
#		define	NS_CONSUMES_SELF		__attribute__((ns_consumes_self))
#	else
#		define	NS_CONSUMES_SELF
#	endif
#endif

#ifndef	CLANG_ANALYZER_NORETURN
#	if	__has_feature(attribute_analyzer_noreturn)
#		define	CLANG_ANALYZER_NORETURN	__attribute__((analyzer_noreturn))
#	else
#		define	CLANG_ANALYZER_NORETURN
#	endif
#endif
